@@ -33,8 +33,8 @@ import (
 
 	kubeclient "k8s.io/client-go/kubernetes"
 	corev1listers "k8s.io/client-go/listers/core/v1"
+	"knative.dev/networking/pkg/apis/networking/v1alpha1"
 	"knative.dev/pkg/tracker"
-	"knative.dev/serving/pkg/apis/networking/v1alpha1"
 
 	v2 "github.com/envoyproxy/go-control-plane/envoy/api/v2"
 	route "github.com/envoyproxy/go-control-plane/envoy/api/v2/route"
@@ -255,10 +255,27 @@ func createRouteForRevision(ingressName string, ingressNamespace string, httpPat
 	}
 
 	return envoy.NewRoute(
-		routeName, path, wrs, routeTimeout, uint32(attempts), perTryTimeout, httpPath.AppendHeaders,
+		routeName, matchHeadersFromHTTPPath(httpPath), path, wrs, routeTimeout, uint32(attempts), perTryTimeout, httpPath.AppendHeaders,
 	)
 }
 
+func matchHeadersFromHTTPPath(httpPath v1alpha1.HTTPIngressPath) []*route.HeaderMatcher {
+	matchHeaders := make([]*route.HeaderMatcher, 0, len(httpPath.Headers))
+
+	for header, matchType := range httpPath.Headers {
+		matchHeader := route.HeaderMatcher{
+			Name: header,
+		}
+		if matchType.Exact != "" {
+			matchHeader.HeaderMatchSpecifier = &route.HeaderMatcher_ExactMatch{
+				ExactMatch: matchType.Exact,
+			}
+		}
+		matchHeaders = append(matchHeaders, &matchHeader)
+	}
+	return matchHeaders
+}
+
 func sniMatchFromIngressTLS(ingressTLS v1alpha1.IngressTLS, kubeClient kubeclient.Interface) (*envoy.SNIMatch, error) {
 	certChain, privateKey, err := sslCreds(
 		kubeClient, ingressTLS.SecretNamespace, ingressTLS.SecretName,
