@@ -39,7 +39,7 @@ func readyRoute() *route.Route {
 	cluster := envoy.NewWeightedCluster("service_stats", 100, map[string]string{})
 	var wrs []*route.WeightedCluster_ClusterWeight
 	wrs = append(wrs, cluster)
-	route := envoy.NewRoute("gateway_ready", "/ready", wrs, 1*time.Second, 0, 5*time.Second, map[string]string{})
+	route := envoy.NewRoute("gateway_ready", nil, "/ready", wrs, 1*time.Second, 0, 5*time.Second, map[string]string{})
 
 	return route
 }
